#include <Adafruit_ADS1015.h>
#include <ESP8266WiFi.h>          //https://github.com/esp8266/Arduino
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <math.h>
#include "WiFiManager.h"          //https://github.com/tzapu/WiFiManager
#include <ESP8266mDNS.h>;
#include <EEPROM.h>
#include <Wire.h>

#include "index.h"

#define Taster1 12
#define Taster2 13
#define Relais1 5
#define Relais2 4

WiFiServer server(80);          // starte WIFI server
Adafruit_ADS1115 ads(0x48);     // verbinde ADDR pin mit GND

String HTTP_req;                // speichert den HTTP request
String html_2, html_4;
int Taster1_alt = 0;
int Taster2_alt = 0;
int temp1, temp2, adc0, count, offset, power;
float factor;

void setup(void)                    // wird nur einmal während des Starts augeführt
{
  pinMode(Relais1, OUTPUT);
  pinMode(Relais2, OUTPUT);
  pinMode(Taster1, INPUT_PULLUP);
  pinMode(Taster2, INPUT_PULLUP);
  digitalWrite(Relais1, LOW); // alle Relais ausschalten
  digitalWrite(Relais2, LOW); // alle Relais ausschalten

  Serial.begin(115200);
  Serial.println("\n Start...");
  WiFi.printDiag(Serial);       // entferne diese Zeile um das WIFI Passwort nicht anzuzeigen
  Serial.println("Opening configuration portal");  

  Wire.begin(2,14);             //2 ist SDA and 14 ist SCL
  // AD Converter Einstellungen 
  // The ADC input range (or gain) can be changed via the following
  // functions, but be careful never to exceed VDD +0.3V max, or to
  // exceed the upper and lower limits if you adjust the input range!
  // Setting these values incorrectly may destroy your ADC!
  // ads.setGain(GAIN_TWOTHIRDS); factor = 0.4313; // Imax = 33 A    2/3x gain +/- 6.144V  1 bit = 3mV      0.1875mV (default)
  // ads.setGain(GAIN_ONE);       factor = 0.2875; // Imax = 33 A    1x gain   +/- 4.096V  1 bit = 2mV      0.125mV
  // ads.setGain(GAIN_TWO);       factor = 0.1438; // Imax = 20,5 A  2x gain   +/- 2.048V  1 bit = 1mV      0.0625mV
     ads.setGain(GAIN_FOUR);      factor = 0.0719; // Imax = 10,2 A  4x gain   +/- 1.024V  1 bit = 0.5mV    0.03125mV
  // ads.setGain(GAIN_EIGHT);     factor = 0.0359; // Imax = 5,12 A  8x gain   +/- 0.512V  1 bit = 0.25mV   0.015625mV
  // ads.setGain(GAIN_SIXTEEN);   factor = 0.0180; // Imax = 2,56 A  16x gain  +/- 0.256V  1 bit = 0.125mV  0.0078125mV
  // factor = +Umax / 32767 * 10 * 230 V
  ads.begin();
  for (int i=0; i <= 9; i++)          // offset Wert ermitteln
  {
    offset += ads.readADC_SingleEnded(0); // 10 mal den Werte lesen
    delay(10);
  }
  offset = offset / 10;               // Mittelung der 10 Werte
  Serial.println((String)"Offset: "+offset);

  //EEPROM, wird im Flash emuliert da kein richtiges EEPROM vorhanden ist
  EEPROM.begin(32);                   // 32 bytes werden reserviert im EEPROM
  if (EEPROM.read(Relais1) == 0x01)
    { digitalWrite(Relais1, HIGH); }
  else
    { digitalWrite(Relais1, LOW); }
  delay(1000);                        // Ausgänge verzögert einschalten
  if (EEPROM.read(Relais2) == 0x01)
    { digitalWrite(Relais2, HIGH); }
  else
    { digitalWrite(Relais2, LOW); }
  
  //WiFiManager
  //Intialisierung 
  WiFiManager wifiManager;
  wifiManager.autoConnect("Netzwerkschalter","passwort"); // Passwort muss mindestens 8 Buchstaben enthalten
  
  //reset settings - for testing
  //wifiManager.resetSettings();

  // if you get here you have connected to the WiFi
  Serial.println("Connected.");  
  
  //set callback that gets called when connecting to previous WiFi fails, and enters Access Point mode
  //wifiManager.setAPCallback(configModeCallback);

  if (!MDNS.begin("nws"))       // hier den Namen eingeben für den Zugriff via "xxx.local" im Browser
    { Serial.println("Error setting up MDNS responder!"); }
  else
    { Serial.println("mDNS responder started"); }
 
  // Starten des Servers
  server.begin();
  Serial.println("Server started");

  // Add service to MDNS-SD
  //  MDNS.addService("http", "tcp", 80);

}

void loop()
{
    if ((digitalRead(Taster1) == LOW) && (temp1 == 0))        // Taster 1 auslesen
    {
      delay(20);
      temp1 = 1;                                              // Dauerschleife vermeiden
      if (digitalRead(Relais1) == HIGH)                       // Ausgang invertieren
        { digitalWrite(Relais1, LOW); }
      else if (digitalRead(Relais1) == LOW)
        { digitalWrite(Relais1, HIGH); }
    }
    else if ((digitalRead(Taster1) == HIGH) && (temp1 == 1))  // Taster nach drücken losgelassen?
    {
      delay(20);
      temp1 = 0;
      if (digitalRead(Relais1) == HIGH)
      {
        EEPROM.write(Relais1, 0x01);
        EEPROM.commit();                      // wird nur für ESP8266 benötigt
      }
      else
      {
        EEPROM.write(Relais1, 0x00);
        EEPROM.commit();                      // wird nur für ESP8266 benötigt
      }
    }

    if ((digitalRead(Taster2) == LOW) && (temp2 == 0))
    {
      delay(20);
      temp2 = 1;                                              // Dauerschleife vermeiden
      if (digitalRead(Relais2) == HIGH)                       // Ausgang invertieren
        { digitalWrite(Relais2, LOW); }
      else if (digitalRead(Relais2) == LOW)
        { digitalWrite(Relais2, HIGH); }
    }
    else if ((digitalRead(Taster2) == HIGH) && (temp2 == 1))  // Taster nach drücken losgelassen?
    {
      delay(20);
      temp2 = 0;
      if (digitalRead(Relais2) == HIGH)
      {
        EEPROM.write(Relais2, 0x01);
        EEPROM.commit();                        // wird nur für ESP8266 benötigt
      }
      else
      {
        EEPROM.write(Relais2, 0x00);
        EEPROM.commit();                        // wird nur für ESP8266 benötigt
      }
    }

    MDNS.update();
    // Check if a client has connected
    WiFiClient client = server.available();

    if (client)                                 // client verbunden?
    {  
        boolean currentLineIsBlank = true;
        while (client.connected()) 
        {
            if (client.available())             // client Daten vorhanden zum lesen
            {   
                char c = client.read();         // read 1 byte (character) from client
                HTTP_req += c;                  // speichere the HTTP request 1 char nach dem anderen 
                // Die letzte Zeiel des client request is leer und ended mit \n
                // Antworten nach dem alles empfangen wurde
                if (c == '\n' && currentLineIsBlank) 
                {
                    adc0 = ads.readADC_SingleEnded(0) - offset; // offset subtrahieren
                    if (adc0 < 0)                               // Wenn Vin < 0V dann 0 V
                      { adc0 = 0; }
                    power = round(factor * adc0);               // Leistung auf ganze Zahl runden 
                    Serial.println((String)"AIN0: "+adc0+" Leistung: "+(power));  // ADC Wert und Leistung seriell ausgeben
                    Serial.print(HTTP_req);
                    client.println( header );                   // sende ein Standard http header
                    // AJAX request for switch state
                    if (HTTP_req.indexOf("ajax_switch") > -1)   // Auswerten der erhaltenen Informationen von der Weboberfläche
                    {
                      if ( HTTP_req.indexOf("LED1=1") > -1 )    // Ausgang schalten und im EEPROM speichern
                      { 
                          digitalWrite(Relais1, HIGH); 
                          EEPROM.write(Relais1, 0x01);
                          EEPROM.commit();                      // wird nur für ESP8266 benötigt 
                      }
                      else if ( HTTP_req.indexOf("LED1=0") > -1 )// Ausgang schalten und im EEPROM speichern
                      {
                          digitalWrite(Relais1, LOW); 
                          EEPROM.write(Relais1, 0x00);
                          EEPROM.commit();                      // wird nur für ESP8266 benötigt
                      }
                  
                      if ( HTTP_req.indexOf("LED2=1") > -1 )    // Ausgang schalten und im EEPROM speichern
                      {
                          digitalWrite(Relais2, HIGH);
                          EEPROM.write(Relais2, 0x01);
                          EEPROM.commit();                      // wird nur für ESP8266 benötigt
                      }
                      else if ( HTTP_req.indexOf("LED2=0") > -1 )// Ausgang schalten und im EEPROM speichern
                      {
                          digitalWrite(Relais2, LOW); 
                          EEPROM.write(Relais2, 0x00);
                          EEPROM.commit();                      // wird nur für ESP8266 benötigt
                      }
                      // senden des aktuellen Relais Status an den Browser. Übertragen nur der Änderungen 
                      if (digitalRead(Relais1)) 
                        { client.println ( R"=====(   <td> <input type='button' class="btn_on" style="color:white "id="LED1_button" value='An' onclick='GetButton(this.id,1)' />  </td><br>)=====" ); }
                      else
                        { client.println ( R"=====(   <td> <input type='button' class="btn_off" style="color:white" id="LED1_button" value='Aus' onclick='GetButton(this.id,1)' />  </td><br>)=====" ); }
                      if (digitalRead(Relais2)) 
                        { client.println( R"=====(    <td> <input type='button' class="btn_on" style="color:white" id="LED2_button" value='An' onclick='GetButton(this.id,2)' />  </td><br>)=====" ); }
                      else
                        { client.println( R"=====(    <td> <input type='button' class="btn_off" style="color:white" id="LED2_button" value='Aus' onclick='GetButton(this.id,2)' />  </td><br>)=====" ); }
                      client.println("<td>"+String(power)+" W</td><br>");        // Ausgabe der Leistung via html
                    }
                    else 
                    {  // HTTP request für die komplette Seite
                          client.println( html_1 );           // komplette HTML Seite hochladen
                    }
                    HTTP_req = "";                            // HTTP request beendet, String leeren
                    break;
                }
                // Jede empfangene Textzeile vom Client ended mit \r\n
                if (c == '\n')
                {
                    // Letztes Zeichen des empfangenen Texts
                    // Neue Zeile beginnen und weitere Zeichen lesen
                    currentLineIsBlank = true;
                } 
                else if (c != '\r') 
                {
                    // Ein Zeichen wurde vom Client emfpangen
                    currentLineIsBlank = false;
                }
            } // end if (client.available())
        } // end while (client.connected())
        delay(1);      // Dem Browser Zeit geben die Daten zu empfangen
        client.stop(); // Verbindung beenden 
    } // end if (client)
}
